''' Fetch data from pretalx '''
from typing import List
from requests import Session
from pydantic import BaseModel


class Item(BaseModel):
    ''' Item '''
    code: str
    title: str
    track_id: int
    state: str
    abstract: str


class PretalxResponse(BaseModel):
    ''' PretalxResponse '''
    count: int
    next: str | None
    previous: str | None
    results: List[Item]


class Pretalx(Session):
    ''' Pretalx '''

    def __init__(self, domain: str, project: str, resource: str) -> None:
        super().__init__()
        self.url = f'https://{domain}/api/events/{project}/{resource}/'

    def fetch(self) -> PretalxResponse:
        ''' Fetch data '''
        return PretalxResponse.parse_obj(self.get(url=self.url, params={'limit': 1000}).json())



''' __init__ '''
__version__ = '0.3.0'
from .openai import Choice
from .openai import Message
from .openai import OpenAIAPI
from .openai import RespCompletions
from .openai import Role
from .openai import TokenUsage
from .pretalx import Item
from .pretalx import Pretalx
from .pretalx import PretalxResponse
from .utils import DATA2022
from .utils import DATA2023

__all__ = [
    "__version__",
    "Choice",
    "Message",
    "OpenAIAPI",
    "RespCompletions",
    "Role",
    "TokenUsage",
    "Item",
    "Pretalx",
    "PretalxResponse",
    "DATA2022",
    "DATA2023",
]



''' Utils '''
DATA2022 = {
    'domain': 'pretalx.com',
    'project': 'coscup-2022',
    'resource': 'talks',
}

DATA2023 = {
    'domain': 'pretalx.coscup.org',
    'project': 'coscup-2023-call-for-participation',
    'resource': 'talks',
}



''' OpenAI API Connect '''
from enum import Enum
from typing import List
from requests import Session
from pydantic import BaseModel


class Role(str, Enum):
    ''' Role '''
    ASSISTANT = 'assistant'
    SYSTEM = 'system'
    USER = 'user'


class Message(BaseModel):
    ''' Message in chat format '''
    role: Role
    content: str

    class Config:
        use_enum_values = True


class TokenUsage(BaseModel):
    ''' Token usage '''
    completion_tokens: int
    prompt_tokens: int
    total_tokens: int


class Choice(BaseModel):
    ''' choice '''
    finish_reason: str
    index: int
    message: Message


class RespCompletions(BaseModel):
    ''' Response of chat.completions'''
    id: str
    model: str
    object: str
    created: int
    usage: TokenUsage
    choices: list[Choice]


class OpenAIAPI(Session):
    ''' OpenAI API '''

    def __init__(self, token: str, organization: str | None) -> None:
        super().__init__()
        self.url = 'https://api.openai.com/v1'
        self.headers['Authorization'] = f'Bearer {token}'

        if organization:
            self.headers['OpenAI-Organization'] = organization

    def chat_completions(self,
                         messages: List[Message],
                         model: str = 'gpt-3.5-turbo',
                         temperature: int = 1,
                         n: int = 1,
                         user: str = 'api') -> RespCompletions:
        ''' chat completions '''
        data = {
            'model': model,
            'messages': [msg.dict() for msg in messages],
            'temperature': temperature,
            'n': n,
            'user': user,
        }
        return RespCompletions.parse_obj(
            self.post(self.url + '/chat/completions', json=data).json()
        )
